@@ -14,6 +14,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import copy
 import datetime
 import re
 
@@ -47,12 +48,16 @@ FAKE_IMAGE_UUID_1 = 'c99d7632-bd66-4be9-aed5-3dd14b223a76'
 FAKE_IMAGE_UUID_2 = 'f27f479a-ddda-419a-9bbc-d6b56b210161'
 FAKE_IMAGE_UUID_SNAPSHOT = '555cae93-fb41-4145-9c52-f5b923538a26'
 FAKE_IMAGE_UUID_SNAP_DEL = '55bb23af-97a4-4068-bdf8-f10c62880ddf'
+FAKE_IMAGE_UUID_BACKUP = '2f87e889-41a4-4778-8553-83f5eea68c5d'
 
 # fake request id
 FAKE_REQUEST_ID = fakes.FAKE_REQUEST_ID
 FAKE_REQUEST_ID_LIST = fakes.FAKE_REQUEST_ID_LIST
 FAKE_RESPONSE_HEADERS = {'x-openstack-request-id': FAKE_REQUEST_ID}
 
+FAKE_SERVICE_UUID_1 = '75e9eabc-ed3b-4f11-8bba-add1e7e7e2de'
+FAKE_SERVICE_UUID_2 = '1f140183-c914-4ddf-8757-6df73028aa86'
+
 
 class FakeClient(fakes.FakeClient, client.Client):
 
@@ -682,7 +687,7 @@ class FakeSessionClient(base_client.SessionClient):
     # Server actions
     #
 
-    none_actions = ['revertResize', 'migrate', 'os-stop', 'os-start',
+    none_actions = ['revertResize', 'os-stop', 'os-start',
                     'forceDelete', 'restore', 'pause', 'unpause', 'unlock',
                     'unrescue', 'resume', 'suspend', 'lock', 'shelve',
                     'shelveOffload', 'unshelve', 'resetNetwork']
@@ -699,13 +704,6 @@ class FakeSessionClient(base_client.SessionClient):
             assert 'flavorRef' in body[action]
         elif action in cls.none_actions:
             assert body[action] is None
-        elif action == 'addFixedIp':
-            assert list(body[action]) == ['networkId']
-        elif action in ['removeFixedIp', 'removeFloatingIp']:
-            assert list(body[action]) == ['address']
-        elif action == 'addFloatingIp':
-            assert (list(body[action]) == ['address'] or
-                    sorted(list(body[action])) == ['address', 'fixed_address'])
         elif action == 'changePassword':
             assert list(body[action]) == ['adminPass']
         elif action in cls.type_actions:
@@ -716,9 +714,6 @@ class FakeSessionClient(base_client.SessionClient):
             assert body[action] is None
         elif action in ['addSecurityGroup', 'removeSecurityGroup']:
             assert list(body[action]) == ['name']
-        elif action == 'createBackup':
-            assert set(body[action]) == set(['name', 'backup_type',
-                                             'rotation'])
         elif action == 'trigger_crash_dump':
             assert body[action] is None
         else:
@@ -747,6 +742,15 @@ class FakeSessionClient(base_client.SessionClient):
             if self.api_version < api_versions.APIVersion("2.25"):
                 expected.add('disk_over_commit')
             assert set(body[action].keys()) == expected
+        elif action == 'migrate':
+            if self.api_version < api_versions.APIVersion("2.56"):
+                assert body[action] is None
+            else:
+                expected = set()
+                if 'host' in body[action].keys():
+                    # host can be optional
+                    expected.add('host')
+                assert set(body[action].keys()) == expected
         elif action == 'rebuild':
             body = body[action]
             adminPass = body.get('adminPass', 'randompassword')
@@ -766,11 +770,22 @@ class FakeSessionClient(base_client.SessionClient):
             _body = {'adminPass': 'RescuePassword'}
         elif action == 'createImage':
             assert set(body[action].keys()) == set(['name', 'metadata'])
-            _headers = dict(location="http://blah/images/%s" %
-                            FAKE_IMAGE_UUID_SNAPSHOT)
+            if self.api_version < api_versions.APIVersion('2.45'):
+                _headers = dict(location="http://blah/images/%s" %
+                                FAKE_IMAGE_UUID_SNAPSHOT)
+            else:
+                _body = {'image_id': FAKE_IMAGE_UUID_SNAPSHOT}
             if body[action]['name'] == 'mysnapshot_deleted':
                 _headers = dict(location="http://blah/images/%s" %
                                 FAKE_IMAGE_UUID_SNAP_DEL)
+        elif action == 'createBackup':
+            assert set(body[action].keys()) == set(['name', 'backup_type',
+                                                    'rotation'])
+            if self.api_version < api_versions.APIVersion('2.45'):
+                _headers = dict(location="http://blah/images/%s" %
+                                FAKE_IMAGE_UUID_BACKUP)
+            else:
+                _body = {'image_id': FAKE_IMAGE_UUID_BACKUP}
         elif action == 'os-getConsoleOutput':
             assert list(body[action]) == ['length']
             return (202, {}, {'output': 'foo'})
@@ -793,36 +808,18 @@ class FakeSessionClient(base_client.SessionClient):
     def post_servers_5678_action(self, body, **kw):
         return self.post_servers_1234_action(body, **kw)
 
-    #
-    # Cloudpipe
-    #
-
-    def get_os_cloudpipe(self, **kw):
-        return (
-            200,
-            {},
-            {'cloudpipes': [{'project_id': 1}]}
-        )
-
-    def post_os_cloudpipe(self, **ks):
-        return (
-            202,
-            {},
-            {'instance_id': '9d5824aa-20e6-4b9f-b967-76a699fc51fd'}
-        )
-
-    def put_os_cloudpipe_configure_project(self, **kw):
-        return (202, {}, None)
-
     #
     # Flavors
     #
 
     def get_flavors(self, **kw):
         status, header, flavors = self.get_flavors_detail(**kw)
+        included_fields = ['id', 'name']
+        if self.api_version >= api_versions.APIVersion('2.55'):
+            included_fields.append('description')
         for flavor in flavors['flavors']:
             for k in list(flavor):
-                if k not in ['id', 'name']:
+                if k not in included_fields:
                     del flavor[k]
 
         return (200, FAKE_RESPONSE_HEADERS, flavors)
@@ -868,6 +865,18 @@ class FakeSessionClient(base_client.SessionClient):
                     if not v['os-flavor-access:is_public']
                 ]
 
+        # Add description in the response for all flavors.
+        if self.api_version >= api_versions.APIVersion('2.55'):
+            for flavor in flavors['flavors']:
+                flavor['description'] = None
+            # Add a new flavor that is a copy of the first but with a different
+            # name, flavorid and a description set.
+            new_flavor = copy.deepcopy(flavors['flavors'][0])
+            new_flavor['id'] = 'with-description'
+            new_flavor['name'] = 'with-description'
+            new_flavor['description'] = 'test description'
+            flavors['flavors'].append(new_flavor)
+
         return (200, FAKE_RESPONSE_HEADERS, flavors)
 
     def get_flavors_1(self, **kw):
@@ -925,6 +934,14 @@ class FakeSessionClient(base_client.SessionClient):
                 self.get_flavors_detail(is_public='None')[2]['flavors'][2]}
         )
 
+    def get_flavors_with_description(self, **kw):
+        return (
+            200,
+            {},
+            {'flavor':
+                self.get_flavors_detail(is_public='None')[2]['flavors'][-1]}
+        )
+
     def delete_flavors_flavordelete(self, **kw):
         return (202, FAKE_RESPONSE_HEADERS, None)
 
@@ -939,6 +956,14 @@ class FakeSessionClient(base_client.SessionClient):
                 self.get_flavors_detail(is_public='None')[2]['flavors'][0]}
         )
 
+    def put_flavors_with_description(self, body, **kw):
+        assert 'flavor' in body
+        assert 'description' in body['flavor']
+        flavor = self.get_flavors_with_description(**kw)[2]
+        # Fake out the actual update of the flavor description for the response
+        flavor['description'] = body['flavor']['description']
+        return (200, {}, {'flavor': flavor})
+
     def get_flavors_1_os_extra_specs(self, **kw):
         return (
             200,
@@ -996,114 +1021,10 @@ class FakeSessionClient(base_client.SessionClient):
         return (202, FAKE_RESPONSE_HEADERS,
                 self.get_flavors_2_os_flavor_access()[2])
 
-    #
-    # Floating IPs
-    #
-
-    def get_os_floating_ips(self, **kw):
-        return (
-            200,
-            {},
-            {'floating_ips': [
-                {'id': 1, 'fixed_ip': '10.0.0.1', 'ip': '11.0.0.1'},
-                {'id': 2, 'fixed_ip': '10.0.0.2', 'ip': '11.0.0.2'},
-            ]},
-        )
-
-    def get_os_floating_ips_1(self, **kw):
-        return (
-            200, {}, {'floating_ip': {'id': 1, 'fixed_ip': '10.0.0.1',
-                                      'ip': '11.0.0.1'}})
-
-    def post_os_floating_ips(self, body):
-        if body.get('pool'):
-            return (
-                200, {}, {'floating_ip': {'id': 1, 'fixed_ip': '10.0.0.1',
-                                          'ip': '11.0.0.1',
-                                          'pool': 'nova'}})
-        else:
-            return (
-                200, {}, {'floating_ip': {'id': 1, 'fixed_ip': '10.0.0.1',
-                                          'ip': '11.0.0.1',
-                                          'pool': None}})
-
-    def delete_os_floating_ips_1(self, **kw):
-        return (204, {}, None)
-
-    def get_os_floating_ip_dns(self, **kw):
-        return (205, {}, {'domain_entries':
-                          [{'domain': 'example.org'},
-                           {'domain': 'example.com'}]})
-
-    def get_os_floating_ip_dns_testdomain_entries(self, **kw):
-        if kw.get('ip'):
-            return (205, {}, {
-                'dns_entries': [
-                    {'dns_entry': {'ip': kw.get('ip'),
-                                   'name': "host1",
-                                   'type': "A",
-                                   'domain': 'testdomain'}},
-                    {'dns_entry': {'ip': kw.get('ip'),
-                                   'name': "host2",
-                                   'type': "A",
-                                   'domain': 'testdomain'}}]})
-        else:
-            return (404, {}, None)
-
-    def get_os_floating_ip_dns_testdomain_entries_testname(self, **kw):
-        return (205, {}, {
-            'dns_entry': {'ip': "10.10.10.10",
-                          'name': 'testname',
-                          'type': "A",
-                          'domain': 'testdomain'}})
-
-    def put_os_floating_ip_dns_testdomain(self, body, **kw):
-        if body['domain_entry']['scope'] == 'private':
-            fakes.assert_has_keys(body['domain_entry'],
-                                  required=['availability_zone', 'scope'])
-        elif body['domain_entry']['scope'] == 'public':
-            fakes.assert_has_keys(body['domain_entry'],
-                                  required=['project', 'scope'])
-
-        else:
-            fakes.assert_has_keys(body['domain_entry'],
-                                  required=['project', 'scope'])
-        return (205, {}, body)
-
-    def put_os_floating_ip_dns_testdomain_entries_testname(self, body, **kw):
-        fakes.assert_has_keys(body['dns_entry'],
-                              required=['ip', 'dns_type'])
-        return (205, {}, body)
-
-    def delete_os_floating_ip_dns_testdomain(self, **kw):
-        return (200, {}, None)
-
-    def delete_os_floating_ip_dns_testdomain_entries_testname(self, **kw):
-        return (200, {}, None)
-
-    def get_os_floating_ips_bulk(self, **kw):
-        return (200, {}, {'floating_ip_info': [
-            {'id': 1, 'fixed_ip': '10.0.0.1', 'ip': '11.0.0.1'},
-            {'id': 2, 'fixed_ip': '10.0.0.2', 'ip': '11.0.0.2'},
-        ]})
-
-    def post_os_floating_ips_bulk(self, **kw):
-        params = kw.get('body').get('floating_ips_bulk_create')
-        pool = params.get('pool', 'defaultPool')
-        interface = params.get('interface', 'defaultInterface')
-        return (200, {}, {'floating_ips_bulk_create':
-                          {'ip_range': '192.168.1.0/30',
-                           'pool': pool,
-                           'interface': interface}})
-
-    def put_os_floating_ips_bulk_delete(self, **kw):
-        ip_range = kw.get('body').get('ip_range')
-        return (200, {}, {'floating_ips_bulk_delete': ip_range})
-
     #
     # Images
     #
-    def get_images_detail(self, **kw):
+    def get_images(self, **kw):
         return (200, {}, {'images': [
             {
                 "id": FAKE_IMAGE_UUID_SNAPSHOT,
@@ -1131,9 +1052,7 @@ class FakeSessionClient(base_client.SessionClient):
                 "updated": "2010-10-10T12:00:00Z",
                 "created": "2010-08-10T12:00:00Z",
                 "status": "ACTIVE",
-                "metadata": {
-                    "test_key": "test_value",
-                },
+                "test_key": "test_value",
                 "links": {},
             },
             {
@@ -1145,45 +1064,37 @@ class FakeSessionClient(base_client.SessionClient):
                 "status": "SAVING",
                 "progress": 80,
                 "links": {},
-            }
+            },
+            {
+                "id": FAKE_IMAGE_UUID_BACKUP,
+                "name": "back1",
+                "serverId": '1234',
+                "updated": "2010-10-10T12:00:00Z",
+                "created": "2010-08-10T12:00:00Z",
+                "status": "SAVING",
+                "progress": 80,
+                "links": {},
+            },
         ]})
 
     def get_images_555cae93_fb41_4145_9c52_f5b923538a26(self, **kw):
-        return (200, {}, {'image': self.get_images_detail()[2]['images'][0]})
+        return (200, {}, {'image': self.get_images()[2]['images'][0]})
 
     def get_images_55bb23af_97a4_4068_bdf8_f10c62880ddf(self, **kw):
-        return (200, {}, {'image': self.get_images_detail()[2]['images'][1]})
+        return (200, {}, {'image': self.get_images()[2]['images'][1]})
 
     def get_images_c99d7632_bd66_4be9_aed5_3dd14b223a76(self, **kw):
-        return (200, {}, {'image': self.get_images_detail()[2]['images'][2]})
+        return (200, {}, {'image': self.get_images()[2]['images'][2]})
 
     def get_images_f27f479a_ddda_419a_9bbc_d6b56b210161(self, **kw):
-        return (200, {}, {'image': self.get_images_detail()[2]['images'][3]})
+        return (200, {}, {'image': self.get_images()[2]['images'][3]})
+
+    def get_images_2f87e889_41a4_4778_8553_83f5eea68c5d(self, **kw):
+        return (200, {}, {'image': self.get_images()[2]['images'][4]})
 
     def get_images_3e861307_73a6_4d1f_8d68_f68b03223032(self):
         raise exceptions.NotFound('404')
 
-    def post_images_c99d7632_bd66_4be9_aed5_3dd14b223a76_metadata(
-            self, body, **kw):
-        assert list(body) == ['metadata']
-        fakes.assert_has_keys(body['metadata'],
-                              required=['test_key'])
-        get_image = self.get_images_c99d7632_bd66_4be9_aed5_3dd14b223a76
-        return (
-            200,
-            {},
-            {'metadata': get_image()[2]['image']['metadata']})
-
-    def delete_images_c99d7632_bd66_4be9_aed5_3dd14b223a76(self, **kw):
-        return (204, {}, None)
-
-    def delete_images_f27f479a_ddda_419a_9bbc_d6b56b210161(self, **kw):
-        return (204, {}, None)
-
-    def delete_images_c99d7632_bd66_4be9_aed5_3dd14b223a76_metadata_test_key(
-            self, **kw):
-        return (204, {}, None)
-
     #
     # Keypairs
     #
@@ -1216,14 +1127,6 @@ class FakeSessionClient(base_client.SessionClient):
         r = {'keypair': self.get_os_keypairs()[2]['keypairs'][0]['keypair']}
         return (202, {}, r)
 
-    #
-    # Virtual Interfaces
-    #
-    def get_servers_1234_os_virtual_interfaces(self, **kw):
-        return (200, {}, {"virtual_interfaces": [
-            {'id': 'fakeid', 'mac_address': 'fakemac'}
-        ]})
-
     #
     # Quotas
     #
@@ -1394,6 +1297,20 @@ class FakeSessionClient(base_client.SessionClient):
     #
 
     def get_os_quota_class_sets_test(self, **kw):
+        if self.api_version >= api_versions.APIVersion('2.50'):
+            return (200, FAKE_RESPONSE_HEADERS, {
+                'quota_class_set': {
+                    'id': 'test',
+                    'metadata_items': 1,
+                    'injected_file_content_bytes': 1,
+                    'injected_file_path_bytes': 1,
+                    'ram': 1,
+                    'instances': 1,
+                    'injected_files': 1,
+                    'cores': 1,
+                    'key_pairs': 1,
+                    'server_groups': 1,
+                    'server_group_members': 1}})
         return (200, FAKE_RESPONSE_HEADERS, {
             'quota_class_set': {
                 'id': 'test',
@@ -1402,6 +1319,7 @@ class FakeSessionClient(base_client.SessionClient):
                 'injected_file_path_bytes': 1,
                 'ram': 1,
                 'floating_ips': 1,
+                'fixed_ips': -1,
                 'instances': 1,
                 'injected_files': 1,
                 'cores': 1,
@@ -1411,6 +1329,19 @@ class FakeSessionClient(base_client.SessionClient):
 
     def put_os_quota_class_sets_test(self, body, **kw):
         assert list(body) == ['quota_class_set']
+        if self.api_version >= api_versions.APIVersion('2.50'):
+            return (200, {}, {
+                'quota_class_set': {
+                    'metadata_items': 1,
+                    'injected_file_content_bytes': 1,
+                    'injected_file_path_bytes': 1,
+                    'ram': 1,
+                    'instances': 1,
+                    'injected_files': 1,
+                    'cores': 1,
+                    'key_pairs': 1,
+                    'server_groups': 1,
+                    'server_group_members': 1}})
         return (200, {}, {
             'quota_class_set': {
                 'metadata_items': 1,
@@ -1418,6 +1349,7 @@ class FakeSessionClient(base_client.SessionClient):
                 'injected_file_path_bytes': 1,
                 'ram': 1,
                 'floating_ips': 1,
+                'fixed_ips': -1,
                 'instances': 1,
                 'injected_files': 1,
                 'cores': 1,
@@ -1442,105 +1374,6 @@ class FakeSessionClient(base_client.SessionClient):
                 'security_groups': 1,
                 'security_group_rules': 1}})
 
-    #
-    # Security Groups
-    #
-    def get_os_security_groups(self, **kw):
-        return (200, {}, {"security_groups": [
-            {"name": "test",
-             "description": "FAKE_SECURITY_GROUP",
-             "tenant_id": "4ffc664c198e435e9853f2538fbcd7a7",
-             "id": 1,
-             "rules": [
-                 {"id": 11,
-                  "group": {},
-                  "ip_protocol": "TCP",
-                  "from_port": 22,
-                  "to_port": 22,
-                  "parent_group_id": 1,
-                  "ip_range":
-                      {"cidr": "10.0.0.0/8"}},
-                 {"id": 12,
-                  "group": {
-                      "tenant_id":
-                          "272bee4c1e624cd4a72a6b0ea55b4582",
-                      "name": "test2"},
-
-                  "ip_protocol": "TCP",
-                  "from_port": 222,
-                  "to_port": 222,
-                  "parent_group_id": 1,
-                  "ip_range": {}},
-                 {"id": 14,
-                  "group": {
-                      "tenant_id":
-                          "272bee4c1e624cd4a72a6b0ea55b4582",
-                      "name": "test4"},
-
-                  "ip_protocol": "TCP",
-                  "from_port": -1,
-                  "to_port": -1,
-                  "parent_group_id": 1,
-                  "ip_range": {}}]},
-            {"name": "test2",
-             "description": "FAKE_SECURITY_GROUP2",
-             "tenant_id": "272bee4c1e624cd4a72a6b0ea55b4582",
-             "id": 2,
-             "rules": []},
-            {"name": "test4",
-             "description": "FAKE_SECURITY_GROUP4",
-             "tenant_id": "272bee4c1e624cd4a72a6b0ea55b4582",
-             "id": 4,
-             "rules": []}
-        ]})
-
-    def delete_os_security_groups_1(self, **kw):
-        return (202, {}, None)
-
-    def post_os_security_groups(self, body, **kw):
-        assert list(body) == ['security_group']
-        fakes.assert_has_keys(body['security_group'],
-                              required=['name', 'description'])
-        r = {'security_group':
-             self.get_os_security_groups()[2]['security_groups'][0]}
-        return (202, {}, r)
-
-    def put_os_security_groups_1(self, body, **kw):
-        assert list(body) == ['security_group']
-        fakes.assert_has_keys(body['security_group'],
-                              required=['name', 'description'])
-        return (205, {}, body)
-
-    #
-    # Security Group Rules
-    #
-    def get_os_security_group_rules(self, **kw):
-        return (200, {}, {"security_group_rules": [
-            {'id': 1, 'parent_group_id': 1, 'group_id': 2,
-             'ip_protocol': 'TCP', 'from_port': 22, 'to_port': 22,
-             'cidr': '10.0.0.0/8'}
-        ]})
-
-    def delete_os_security_group_rules_11(self, **kw):
-        return (202, {}, None)
-
-    def delete_os_security_group_rules_12(self, **kw):
-        return (202, {}, None)
-
-    def delete_os_security_group_rules_14(self, **kw):
-        return (202, {}, None)
-
-    def post_os_security_group_rules(self, body, **kw):
-        assert list(body) == ['security_group_rule']
-        fakes.assert_has_keys(
-            body['security_group_rule'],
-            required=['parent_group_id'],
-            optional=['group_id', 'ip_protocol', 'from_port',
-                      'to_port', 'cidr'])
-        r = {'security_group_rule':
-             self.get_os_security_group_rules()[2]['security_group_rules'][0]}
-        return (202, {}, r)
-
     #
     # Tenant Usage
     #
@@ -1703,7 +1536,7 @@ class FakeSessionClient(base_client.SessionClient):
     #
 
     def get_os_aggregates(self, *kw):
-        return (200, {}, {"aggregates": [
+        response = (200, {}, {"aggregates": [
             {'id': '1',
              'name': 'test',
              'availability_zone': 'nova1'},
@@ -1714,6 +1547,13 @@ class FakeSessionClient(base_client.SessionClient):
              'name': 'test3',
              'metadata': {'test': "dup", "none_key": "Nine"}},
         ]})
+        # microversion >= 2.41 returns the uuid in the response
+        if self.api_version >= api_versions.APIVersion('2.41'):
+            aggregates = response[2]['aggregates']
+            aggregates[0]['uuid'] = '80785864-087b-45a5-a433-b20eac9b58aa'
+            aggregates[1]['uuid'] = '30827713-5957-4b68-8fd3-ccaddb568c24'
+            aggregates[2]['uuid'] = '9a651b22-ce3f-4a87-acd7-98446ef591c4'
+        return response
 
     def _return_aggregate(self):
         r = {'aggregate': self.get_os_aggregates()[2]['aggregates'][0]}
@@ -1750,6 +1590,12 @@ class FakeSessionClient(base_client.SessionClient):
     def get_os_services(self, **kw):
         host = kw.get('host', 'host1')
         binary = kw.get('binary', 'nova-compute')
+        if self.api_version >= api_versions.APIVersion('2.53'):
+            service_id_1 = FAKE_SERVICE_UUID_1
+            service_id_2 = FAKE_SERVICE_UUID_2
+        else:
+            service_id_1 = 1
+            service_id_2 = 2
         return (200, FAKE_RESPONSE_HEADERS,
                 {'services': [{'binary': binary,
                                'host': host,
@@ -1757,14 +1603,16 @@ class FakeSessionClient(base_client.SessionClient):
                                'status': 'enabled',
                                'state': 'up',
                                'updated_at': datetime.datetime(
-                                   2012, 10, 29, 13, 42, 2)},
+                                   2012, 10, 29, 13, 42, 2),
+                               'id': service_id_1},
                               {'binary': binary,
                                'host': host,
                                'zone': 'nova',
                                'status': 'disabled',
                                'state': 'down',
                                'updated_at': datetime.datetime(
-                                   2012, 9, 18, 8, 3, 38)},
+                                   2012, 9, 18, 8, 3, 38),
+                               'id': service_id_2},
                               ]})
 
     def put_os_services_enable(self, body, **kw):
@@ -1786,9 +1634,22 @@ class FakeSessionClient(base_client.SessionClient):
             'status': 'disabled',
             'disabled_reason': body['disabled_reason']}})
 
+    def put_os_services_75e9eabc_ed3b_4f11_8bba_add1e7e7e2de(
+            self, body, **kw):
+        """This should only be called with microversion >= 2.53."""
+        return (200, FAKE_RESPONSE_HEADERS, {'service': {
+            'host': 'host1',
+            'binary': 'nova-compute',
+            'status': body.get('status', 'enabled'),
+            'disabled_reason': body.get('disabled_reason'),
+            'forced_down': body.get('forced_down', False)}})
+
     def delete_os_services_1(self, **kw):
         return (204, FAKE_RESPONSE_HEADERS, None)
 
+    def delete_os_services_75e9eabc_ed3b_4f11_8bba_add1e7e7e2de(self, **kwarg):
+        return (204, FAKE_RESPONSE_HEADERS, None)
+
     def put_os_services_force_down(self, body, **kw):
         return (200, FAKE_RESPONSE_HEADERS, {'service': {
             'host': body['host'],
@@ -1796,54 +1657,8 @@ class FakeSessionClient(base_client.SessionClient):
             'forced_down': False}})
 
     #
-    # Fixed IPs
+    # Hypervisors
     #
-    def get_os_fixed_ips_192_168_1_1(self, *kw):
-        return (200, {}, {"fixed_ip": {'cidr': '192.168.1.0/24',
-                                       'address': '192.168.1.1',
-                                       'hostname': 'foo',
-                                       'host': 'bar'}})
-
-    def post_os_fixed_ips_192_168_1_1_action(self, body, **kw):
-        return (202, {}, None)
-
-    #
-    # Hosts
-    #
-
-    def get_os_hosts(self, **kw):
-        zone = kw.get('zone', 'nova1')
-        return (200, {}, {'hosts': [{'host': 'host1',
-                                     'service': 'nova-compute',
-                                     'zone': zone},
-                                    {'host': 'host1',
-                                     'service': 'nova-cert',
-                                     'zone': zone}]})
-
-    def put_os_hosts_sample_host_1(self, body, **kw):
-        return (200, {}, {'host': 'sample-host_1',
-                          'status': 'enabled'})
-
-    def put_os_hosts_sample_host_2(self, body, **kw):
-        return (200, {}, {'host': 'sample-host_2',
-                          'maintenance_mode': 'on_maintenance'})
-
-    def put_os_hosts_sample_host_3(self, body, **kw):
-        return (200, {}, {'host': 'sample-host_3',
-                          'status': 'enabled',
-                          'maintenance_mode': 'on_maintenance'})
-
-    def get_os_hosts_sample_host_reboot(self, **kw):
-        return (200, {}, {'host': 'sample_host',
-                          'power_action': 'reboot'})
-
-    def get_os_hosts_sample_host_startup(self, **kw):
-        return (200, {}, {'host': 'sample_host',
-                          'power_action': 'startup'})
-
-    def get_os_hosts_sample_host_shutdown(self, **kw):
-        return (200, {}, {'host': 'sample_host',
-                          'power_action': 'shutdown'})
 
     def get_os_hypervisors(self, **kw):
         return (200, {}, {
@@ -2012,44 +1827,6 @@ class FakeSessionClient(base_client.SessionClient):
 
         return (200, {}, {"networks": networks_by_name[name]})
 
-    def get_os_networks(self, **kw):
-        return (200, {}, {'networks': [{"label": "1", "cidr": "10.0.0.0/24",
-                                        'project_id':
-                                            '4ffc664c198e435e9853f2538fbcd7a7',
-                                        'id': '1', 'vlan': '1234'}]})
-
-    def delete_os_networks_1(self, **kw):
-        return (202, {}, None)
-
-    def post_os_networks(self, **kw):
-        return (202, {}, {'network': kw})
-
-    def post_os_networks_add(self, **kw):
-        return (202, {}, None)
-
-    def post_os_networks_1_action(self, **kw):
-        return (202, {}, None)
-
-    def post_os_networks_2_action(self, **kw):
-        return (202, {}, None)
-
-    def get_os_tenant_networks(self, **kw):
-        return (200, {}, {'networks': [{"label": "1", "cidr": "10.0.0.0/24",
-                                        'project_id':
-                                            '4ffc664c198e435e9853f2538fbcd7a7',
-                                        'id': '1', 'vlan': '1234'}]})
-
-    def get_os_tenant_networks_1(self, **kw):
-        return (200, {}, {'network': {"label": "1", "cidr": "10.0.0.0/24",
-                                      "id": "1"}})
-
-    def post_os_tenant_networks(self, **kw):
-        return (202, {}, {'network': {"label": "new_network1",
-                                      "cidr1": "10.0.1.0/24"}})
-
-    def delete_os_tenant_networks_1(self, **kw):
-        return (202, {}, None)
-
     def get_os_availability_zone_detail(self, **kw):
         return (200, {}, {
             "availabilityZoneInfo": [
@@ -2206,7 +1983,7 @@ class FakeSessionClient(base_client.SessionClient):
         return self.get_os_cells_capacities()
 
     def get_os_migrations(self, **kw):
-        migration = {
+        migration1 = {
             "created_at": "2012-10-29T13:42:02.000000",
             "dest_compute": "compute2",
             "dest_host": "1.2.3.4",
@@ -2221,10 +1998,35 @@ class FakeSessionClient(base_client.SessionClient):
             "updated_at": "2012-10-29T13:42:02.000000"
         }
 
-        if self.api_version >= api_versions.APIVersion("2.23"):
-            migration.update({"migration_type": "live-migration"})
+        migration2 = {
+            "created_at": "2012-10-29T13:42:02.000000",
+            "dest_compute": "compute2",
+            "dest_host": "1.2.3.4",
+            "dest_node": "node2",
+            "id": '1234',
+            "instance_uuid": "instance_id_456",
+            "new_instance_type_id": 2,
+            "old_instance_type_id": 1,
+            "source_compute": "compute1",
+            "source_node": "node1",
+            "status": "Done",
+            "updated_at": "2013-11-50T13:42:02.000000"
+        }
 
-        migrations = {'migrations': [migration]}
+        if self.api_version >= api_versions.APIVersion("2.23"):
+            migration1.update({"migration_type": "live-migration"})
+            migration2.update({"migration_type": "live-migration"})
+
+        migration_list = []
+        instance_uuid = kw.get('instance_uuid', None)
+        if instance_uuid == migration1['instance_uuid']:
+            migration_list.append(migration1)
+        elif instance_uuid == migration2['instance_uuid']:
+            migration_list.append(migration2)
+        elif instance_uuid is None:
+            migration_list.extend([migration1, migration2])
+
+        migrations = {'migrations': migration_list}
 
         return (200, FAKE_RESPONSE_HEADERS, migrations)
 
@@ -2332,6 +2134,15 @@ class FakeSessionClient(base_client.SessionClient):
     def put_servers_1234_tags_tag(self, **kw):
         return (201, {}, None)
 
+    def put_servers_1234_tags_tag1(self, **kw):
+        return (201, {}, None)
+
+    def put_servers_1234_tags_tag2(self, **kw):
+        return (201, {}, None)
+
+    def put_servers_1234_tags_tag3(self, **kw):
+        return (201, {}, None)
+
     def put_servers_1234_tags(self, **kw):
         return (201, {}, None)
 
@@ -2341,73 +2152,17 @@ class FakeSessionClient(base_client.SessionClient):
     def delete_servers_1234_tags_tag(self, **kw):
         return (204, {}, None)
 
-    def delete_servers_1234_tags(self, **kw):
+    def delete_servers_1234_tags_tag1(self, **kw):
         return (204, {}, None)
 
-    def get_os_tenant_networks(self):
-        return (200, FAKE_RESPONSE_HEADERS, {
-            'networks': [{"label": "1", "cidr": "10.0.0.0/24",
-                          'project_id': '4ffc664c198e435e9853f2538fbcd7a7',
-                          'id': '1'}]})
+    def delete_servers_1234_tags_tag2(self, **kw):
+        return (204, {}, None)
 
-    def get_os_tenant_networks_1(self, **kw):
-        return (200, FAKE_RESPONSE_HEADERS, {
-            'network': {"label": "1", "cidr": "10.0.0.0/24",
-                        'project_id': '4ffc664c198e435e9853f2538fbcd7a7',
-                        'id': '1'}})
+    def delete_servers_1234_tags_tag3(self, **kw):
+        return (204, {}, None)
 
-    def post_os_tenant_networks(self, **kw):
-        return (201, FAKE_RESPONSE_HEADERS, {
-            'network': {"label": "1", "cidr": "10.0.0.0/24",
-                        'project_id': '4ffc664c198e435e9853f2538fbcd7a7',
-                        'id': '1'}})
-
-    def delete_os_tenant_networks_1(self, **kw):
-        return (204, FAKE_RESPONSE_HEADERS, None)
-
-    def get_os_baremetal_nodes(self, **kw):
-        return (
-            200, FAKE_RESPONSE_HEADERS, {
-                'nodes': [
-                    {
-                        "id": 1,
-                        "instance_uuid": None,
-                        "interfaces": [],
-                        "cpus": 2,
-                        "local_gb": 10,
-                        "memory_mb": 5,
-                        "pm_address": "2.3.4.5",
-                        "pm_user": "pmuser",
-                        "pm_password": "pmpass",
-                        "prov_mac_address": "aa:bb:cc:dd:ee:ff",
-                        "prov_vlan_id": 1,
-                        "service_host": "somehost",
-                        "terminal_port": 8080,
-                    }
-                ]
-            }
-        )
-
-    def get_os_baremetal_nodes_1(self, **kw):
-        return (
-            200, FAKE_RESPONSE_HEADERS, {
-                'node': {
-                    "id": 1,
-                    "instance_uuid": None,
-                    "pm_address": "1.2.3.4",
-                    "interfaces": [],
-                    "cpus": 2,
-                    "local_gb": 10,
-                    "memory_mb": 5,
-                    "pm_user": "pmuser",
-                    "pm_password": "pmpass",
-                    "prov_mac_address": "aa:bb:cc:dd:ee:ff",
-                    "prov_vlan_id": 1,
-                    "service_host": "somehost",
-                    "terminal_port": 8080,
-                }
-            }
-        )
+    def delete_servers_1234_tags(self, **kw):
+        return (204, {}, None)
 
     def post_os_assisted_volume_snapshots(self, **kw):
         return (202, FAKE_RESPONSE_HEADERS,
