@@ -14,10 +14,7 @@
 migration interface
 """
 
-from six.moves.urllib import parse
-
 from novaclient import base
-from novaclient.i18n import _
 
 
 class Migration(base.Resource):
@@ -28,28 +25,18 @@ class Migration(base.Resource):
 class MigrationManager(base.ManagerWithFind):
     resource_class = Migration
 
-    def list(self, host=None, status=None, cell_name=None):
+    def list(self, host=None, status=None, instance_uuid=None):
         """
         Get a list of migrations.
         :param host: (optional) filter migrations by host name.
         :param status: (optional) filter migrations by status.
-        :param cell_name: (optional) filter migrations for a cell.
         """
         opts = {}
         if host:
             opts['host'] = host
         if status:
             opts['status'] = status
-        if cell_name:
-            self.client.logger.warning(_("Argument 'cell_name' is "
-                                         "deprecated since Pike, and will "
-                                         "be removed in a future release."))
-            opts['cell_name'] = cell_name
+        if instance_uuid:
+            opts['instance_uuid'] = instance_uuid
 
-        # Transform the dict to a sequence of two-element tuples in fixed
-        # order, then the encoded string will be consistent in Python 2&3.
-        new_opts = sorted(opts.items(), key=lambda x: x[0])
-
-        query_string = "?%s" % parse.urlencode(new_opts) if new_opts else ""
-
-        return self._list("/os-migrations%s" % query_string, "migrations")
+        return self._list("/os-migrations", "migrations", filters=opts)
