@@ -20,7 +20,6 @@ Server interface.
 """
 
 import base64
-import warnings
 
 from oslo_utils import encodeutils
 import six
@@ -52,12 +51,6 @@ CONSOLE_TYPE_PROTOCOL_MAPPING = {
     'webmks': 'mks'
 }
 
-ADD_REMOVE_FIXED_FLOATING_DEPRECATION_WARNING = _(
-    'The %s server action API is deprecated as of the 2.44 microversion. This '
-    'API binding will be removed in the first major release after the Nova '
-    '16.0.0 Pike release. Use python-neutronclient or openstacksdk instead.'
-)
-
 
 class Server(base.Resource):
     HUMAN_ID = True
@@ -172,35 +165,6 @@ class Server(base.Resource):
         """
         return self.manager.clear_password(self)
 
-    def add_fixed_ip(self, network_id):
-        """
-        Add an IP address on a network.
-
-        :param network_id: The ID of the network the IP should be on.
-        :returns: An instance of novaclient.base.TupleWithMeta
-        """
-        return self.manager.add_fixed_ip(self, network_id)
-
-    def add_floating_ip(self, address, fixed_address=None):
-        """
-        Add floating IP to an instance
-
-        :param address: The IP address or FloatingIP to add to the instance
-        :param fixed_address: The fixedIP address the FloatingIP is to be
-               associated with (optional)
-        :returns: An instance of novaclient.base.TupleWithMeta
-        """
-        return self.manager.add_floating_ip(self, address, fixed_address)
-
-    def remove_floating_ip(self, address):
-        """
-        Remove floating IP from an instance
-
-        :param address: The IP address or FloatingIP to remove
-        :returns: An instance of novaclient.base.TupleWithMeta
-        """
-        return self.manager.remove_floating_ip(self, address)
-
     def stop(self):
         """
         Stop -- Stop the running server.
@@ -327,6 +291,7 @@ class Server(base.Resource):
         """Diagnostics -- Retrieve server diagnostics."""
         return self.manager.diagnostics(self)
 
+    @api_versions.wraps("2.0", "2.55")
     def migrate(self):
         """
         Migrate a server to a new host.
@@ -335,14 +300,15 @@ class Server(base.Resource):
         """
         return self.manager.migrate(self)
 
-    def remove_fixed_ip(self, address):
+    @api_versions.wraps("2.56")
+    def migrate(self, host=None):
         """
-        Remove an IP address.
+        Migrate a server to a new host.
 
-        :param address: The IP address to remove.
+        :param host: (Optional) The target host.
         :returns: An instance of novaclient.base.TupleWithMeta
         """
-        return self.manager.remove_fixed_ip(self, address)
+        return self.manager.migrate(self, host=host)
 
     def change_password(self, password):
         """
@@ -577,12 +543,21 @@ class Server(base.Resource):
         """
         return self.manager.interface_list(self)
 
+    @api_versions.wraps("2.0", "2.48")
     def interface_attach(self, port_id, net_id, fixed_ip):
         """
         Attach a network interface to an instance.
         """
         return self.manager.interface_attach(self, port_id, net_id, fixed_ip)
 
+    @api_versions.wraps("2.49")
+    def interface_attach(self, port_id, net_id, fixed_ip, tag=None):
+        """
+        Attach a network interface to an instance with an optional tag.
+        """
+        return self.manager.interface_attach(self, port_id, net_id, fixed_ip,
+                                             tag)
+
     def interface_detach(self, port_id):
         """
         Detach a network interface from an instance.
@@ -646,15 +621,15 @@ class SecurityGroup(base.Resource):
 class ServerManager(base.BootingManagerWithFind):
     resource_class = Server
 
-    def _boot(self, resource_url, response_key, name, image, flavor,
+    def _boot(self, response_key, name, image, flavor,
               meta=None, files=None, userdata=None,
-              reservation_id=None, return_raw=False, min_count=None,
+              reservation_id=False, return_raw=False, min_count=None,
               max_count=None, security_groups=None, key_name=None,
               availability_zone=None, block_device_mapping=None,
               block_device_mapping_v2=None, nics=None, scheduler_hints=None,
               config_drive=None, admin_pass=None, disk_config=None,
               access_ip_v4=None, access_ip_v6=None, description=None,
-              **kwargs):
+              tags=None, **kwargs):
         """
         Create (boot) a new server.
         """
@@ -686,7 +661,8 @@ class ServerManager(base.BootingManagerWithFind):
         if meta:
             body["server"]["metadata"] = meta
         if reservation_id:
-            body["server"]["reservation_id"] = reservation_id
+            body["server"]["return_reservation_id"] = reservation_id
+            return_raw = True
         if key_name:
             body["server"]["key_name"] = key_name
         if scheduler_hints:
@@ -786,7 +762,10 @@ class ServerManager(base.BootingManagerWithFind):
         if description:
             body['server']['description'] = description
 
-        return self._create(resource_url, body, response_key,
+        if tags:
+            body['server']['tags'] = tags
+
+        return self._create('/servers', body, response_key,
                             return_raw=return_raw, **kwargs)
 
     def get(self, server):
@@ -808,7 +787,7 @@ class ServerManager(base.BootingManagerWithFind):
             match the search_opts (optional). The search opts format is a
             dictionary of key / value pairs that will be appended to the query
             string.  For a complete list of keys see:
-            http://developer.openstack.org/api-ref-compute-v2.1.html#listServers
+            https://developer.openstack.org/api-ref/compute/#list-servers
         :param marker: Begin returning servers that appear later in the server
                        list than that represented by this server id (optional).
         :param limit: Maximum number of servers to return (optional).
@@ -878,70 +857,6 @@ class ServerManager(base.BootingManagerWithFind):
             marker = result[-1].id
         return result
 
-    @api_versions.wraps('2.0', '2.43')
-    def add_fixed_ip(self, server, network_id):
-        """
-        DEPRECATED Add an IP address on a network.
-
-        :param server: The :class:`Server` (or its ID) to add an IP to.
-        :param network_id: The ID of the network the IP should be on.
-        :returns: An instance of novaclient.base.TupleWithMeta
-        """
-        warnings.warn(ADD_REMOVE_FIXED_FLOATING_DEPRECATION_WARNING %
-                      'addFixedIP', DeprecationWarning)
-        return self._action('addFixedIp', server, {'networkId': network_id})
-
-    @api_versions.wraps('2.0', '2.43')
-    def remove_fixed_ip(self, server, address):
-        """
-        DEPRECATED Remove an IP address.
-
-        :param server: The :class:`Server` (or its ID) to add an IP to.
-        :param address: The IP address to remove.
-        :returns: An instance of novaclient.base.TupleWithMeta
-        """
-        warnings.warn(ADD_REMOVE_FIXED_FLOATING_DEPRECATION_WARNING %
-                      'removeFixedIP', DeprecationWarning)
-        return self._action('removeFixedIp', server, {'address': address})
-
-    @api_versions.wraps('2.0', '2.43')
-    def add_floating_ip(self, server, address, fixed_address=None):
-        """
-        DEPRECATED Add a floating IP to an instance
-
-        :param server: The :class:`Server` (or its ID) to add an IP to.
-        :param address: The FloatingIP or string floating address to add.
-        :param fixed_address: The FixedIP the floatingIP should be
-                              associated with (optional)
-        :returns: An instance of novaclient.base.TupleWithMeta
-        """
-        warnings.warn(ADD_REMOVE_FIXED_FLOATING_DEPRECATION_WARNING %
-                      'addFloatingIP', DeprecationWarning)
-        address = address.ip if hasattr(address, 'ip') else address
-        if fixed_address:
-            if hasattr(fixed_address, 'ip'):
-                fixed_address = fixed_address.ip
-            return self._action('addFloatingIp', server,
-                                {'address': address,
-                                 'fixed_address': fixed_address})
-        else:
-            return self._action('addFloatingIp', server, {'address': address})
-
-    @api_versions.wraps('2.0', '2.43')
-    def remove_floating_ip(self, server, address):
-        """
-        DEPRECATED Remove a floating IP address.
-
-        :param server: The :class:`Server` (or its ID) to remove an IP from.
-        :param address: The FloatingIP or string floating address to remove.
-        :returns: An instance of novaclient.base.TupleWithMeta
-        """
-        warnings.warn(ADD_REMOVE_FIXED_FLOATING_DEPRECATION_WARNING %
-                      'removeFloatingIP', DeprecationWarning)
-        address = address.ip if hasattr(address, 'ip') else address
-        return self._action('removeFloatingIp', server, {'address': address})
-
-    @api_versions.wraps('2.0', '2.5')
     def get_vnc_console(self, server, console_type):
         """
         Get a vnc console for an instance
@@ -953,7 +868,6 @@ class ServerManager(base.BootingManagerWithFind):
 
         return self.get_console_url(server, console_type)
 
-    @api_versions.wraps('2.0', '2.5')
     def get_spice_console(self, server, console_type):
         """
         Get a spice console for an instance
@@ -965,7 +879,6 @@ class ServerManager(base.BootingManagerWithFind):
 
         return self.get_console_url(server, console_type)
 
-    @api_versions.wraps('2.0', '2.5')
     def get_rdp_console(self, server, console_type):
         """
         Get a rdp console for an instance
@@ -977,7 +890,6 @@ class ServerManager(base.BootingManagerWithFind):
 
         return self.get_console_url(server, console_type)
 
-    @api_versions.wraps('2.0', '2.5')
     def get_serial_console(self, server, console_type):
         """
         Get a serial console for an instance
@@ -1011,54 +923,6 @@ class ServerManager(base.BootingManagerWithFind):
             raise exceptions.UnsupportedConsoleType(console_type)
         return self._action(action, server, {'type': console_type})
 
-    @api_versions.wraps('2.6')
-    def get_vnc_console(self, server, console_type):
-        """
-        Get a vnc console for an instance
-
-        :param server: The :class:`Server` (or its ID) to get console for.
-        :param console_type: Type of vnc console to get ('novnc' or 'xvpvnc')
-        :returns: An instance of novaclient.base.DictWithMeta
-        """
-
-        return self.get_console_url(server, console_type)
-
-    @api_versions.wraps('2.6')
-    def get_spice_console(self, server, console_type):
-        """
-        Get a spice console for an instance
-
-        :param server: The :class:`Server` (or its ID) to get console for.
-        :param console_type: Type of spice console to get ('spice-html5')
-        :returns: An instance of novaclient.base.DictWithMeta
-        """
-
-        return self.get_console_url(server, console_type)
-
-    @api_versions.wraps('2.6')
-    def get_rdp_console(self, server, console_type):
-        """
-        Get a rdp console for an instance
-
-        :param server: The :class:`Server` (or its ID) to get console for.
-        :param console_type: Type of rdp console to get ('rdp-html5')
-        :returns: An instance of novaclient.base.DictWithMeta
-        """
-
-        return self.get_console_url(server, console_type)
-
-    @api_versions.wraps('2.6')
-    def get_serial_console(self, server, console_type):
-        """
-        Get a serial console for an instance
-
-        :param server: The :class:`Server` (or its ID) to get console for.
-        :param console_type: Type of serial console to get ('serial')
-        :returns: An instance of novaclient.base.DictWithMeta
-        """
-
-        return self.get_console_url(server, console_type)
-
     @api_versions.wraps('2.8')
     def get_mks_console(self, server):
         """
@@ -1313,11 +1177,12 @@ class ServerManager(base.BootingManagerWithFind):
         if self.api_version > api_versions.APIVersion('2.36'):
             if not nics:
                 raise ValueError('nics are required after microversion 2.36')
-        elif nics and not isinstance(nics, list):
-            raise ValueError('nics must be a list')
+        elif nics and not isinstance(nics, (list, tuple)):
+            raise ValueError('nics must be a list or a tuple, not %s' %
+                             type(nics))
 
     def create(self, name, image, flavor, meta=None, files=None,
-               reservation_id=None, min_count=None,
+               reservation_id=False, min_count=None,
                max_count=None, security_groups=None, userdata=None,
                key_name=None, availability_zone=None,
                block_device_mapping=None, block_device_mapping_v2=None,
@@ -1339,7 +1204,8 @@ class ServerManager(base.BootingManagerWithFind):
                       are the file contents (either as a string or as a
                       file-like object). A maximum of five entries is allowed,
                       and each file must be 10k or less.
-        :param reservation_id: a UUID for the set of servers being requested.
+        :param reservation_id: return a reservation_id for the set of
+                               servers being requested, boolean.
         :param min_count: (optional extension) The minimum number of
                           servers to launch.
         :param max_count: (optional extension) The maximum number of
@@ -1380,6 +1246,8 @@ class ServerManager(base.BootingManagerWithFind):
         :param access_ip_v6: (optional extension) add alternative access ip v6
         :param description: optional description of the server (allowed since
                             microversion 2.19)
+        :param tags: A list of arbitrary strings to be added to the
+                     server as tags (allowed since microversion 2.52)
         """
         if not min_count:
             min_count = 1
@@ -1412,6 +1280,10 @@ class ServerManager(base.BootingManagerWithFind):
                                          "unsupported before microversion "
                                          "2.32")
 
+        boot_tags_microversion = api_versions.APIVersion("2.52")
+        if "tags" in kwargs and self.api_version < boot_tags_microversion:
+            raise exceptions.UnsupportedAttribute("tags", "2.52")
+
         boot_kwargs = dict(
             meta=meta, files=files, userdata=userdata,
             reservation_id=reservation_id, min_count=min_count,
@@ -1422,19 +1294,15 @@ class ServerManager(base.BootingManagerWithFind):
             access_ip_v4=access_ip_v4, access_ip_v6=access_ip_v6, **kwargs)
 
         if block_device_mapping:
-            resource_url = "/os-volumes_boot"
             boot_kwargs['block_device_mapping'] = block_device_mapping
         elif block_device_mapping_v2:
-            resource_url = "/os-volumes_boot"
             boot_kwargs['block_device_mapping_v2'] = block_device_mapping_v2
-        else:
-            resource_url = "/servers"
+
         if nics:
             boot_kwargs['nics'] = nics
 
-        response_key = "server"
-        return self._boot(resource_url, response_key, *boot_args,
-                          **boot_kwargs)
+        response_key = "server" if not reservation_id else "reservation_id"
+        return self._boot(response_key, *boot_args, **boot_kwargs)
 
     @api_versions.wraps("2.0", "2.18")
     def update(self, server, name=None):
@@ -1531,12 +1399,21 @@ class ServerManager(base.BootingManagerWithFind):
                       and each file must be 10k or less.
         :param description: optional description of the server (allowed since
                             microversion 2.19)
+        :param key_name: optional key pair name for rebuild operation; passing
+                         None will unset the key for the server instance
+                         (starting from microversion 2.54)
         :returns: :class:`Server`
         """
         descr_microversion = api_versions.APIVersion("2.19")
         if "description" in kwargs and self.api_version < descr_microversion:
             raise exceptions.UnsupportedAttribute("description", "2.19")
 
+        # Starting from microversion 2.54,
+        # the optional 'key_name' parameter has been added.
+        if ('key_name' in kwargs and
+                self.api_version < api_versions.APIVersion('2.54')):
+            raise exceptions.UnsupportedAttribute('key_name', '2.54')
+
         body = {'imageRef': base.getid(image)}
         if password is not None:
             body['adminPass'] = password
@@ -1548,6 +1425,8 @@ class ServerManager(base.BootingManagerWithFind):
             body['name'] = name
         if "description" in kwargs:
             body["description"] = kwargs["description"]
+        if 'key_name' in kwargs:
+            body['key_name'] = kwargs['key_name']
         if meta:
             body['metadata'] = meta
         if files:
@@ -1569,6 +1448,7 @@ class ServerManager(base.BootingManagerWithFind):
                                                        body, **kwargs)
         return Server(self, body['server'], resp=resp)
 
+    @api_versions.wraps("2.0", "2.55")
     def migrate(self, server):
         """
         Migrate a server to a new host.
@@ -1578,6 +1458,22 @@ class ServerManager(base.BootingManagerWithFind):
         """
         return self._action('migrate', server)
 
+    @api_versions.wraps("2.56")
+    def migrate(self, server, host=None):
+        """
+        Migrate a server to a new host.
+
+        :param server: The :class:`Server` (or its ID).
+        :param host: (Optional) The target host.
+        :returns: An instance of novaclient.base.TupleWithMeta
+        """
+        info = {}
+
+        if host:
+            info['host'] = host
+
+        return self._action('migrate', server, info)
+
     def resize(self, server, flavor, disk_config=None, **kwargs):
         """
         Resize a server's resources.
@@ -1891,6 +1787,7 @@ class ServerManager(base.BootingManagerWithFind):
         return self._list('/servers/%s/os-interface' % base.getid(server),
                           'interfaceAttachments', obj_class=NetworkInterface)
 
+    @api_versions.wraps("2.0", "2.48")
     def interface_attach(self, server, port_id, net_id, fixed_ip):
         """
         Attach a network_interface to an instance.
@@ -1911,6 +1808,35 @@ class ServerManager(base.BootingManagerWithFind):
         return self._create('/servers/%s/os-interface' % base.getid(server),
                             body, 'interfaceAttachment')
 
+    @api_versions.wraps("2.49")
+    def interface_attach(self, server, port_id, net_id, fixed_ip, tag=None):
+        """
+        Attach a network_interface to an instance.
+
+        :param server: The :class:`Server` (or its ID) to attach to.
+        :param port_id: The port to attach.
+                        The port_id and net_id parameters are mutually
+                        exclusive.
+        :param net_id: The ID of the network to attach.
+        :param fixed_ip: The fixed IP addresses. If the fixed_ip is specified,
+                         the net_id has to be specified at the same time.
+        :param tag: The tag.
+        """
+
+        body = {'interfaceAttachment': {}}
+        if port_id:
+            body['interfaceAttachment']['port_id'] = port_id
+        if net_id:
+            body['interfaceAttachment']['net_id'] = net_id
+        if fixed_ip:
+            body['interfaceAttachment']['fixed_ips'] = [
+                {'ip_address': fixed_ip}]
+        if tag:
+            body['interfaceAttachment']['tag'] = tag
+
+        return self._create('/servers/%s/os-interface' % base.getid(server),
+                            body, 'interfaceAttachment')
+
     def interface_detach(self, server, port_id):
         """
         Detach a network_interface from an instance.
